@@ -252,14 +252,23 @@ class Service(RelationEndpoint):
 class Relation(Base):
 
     @property
-    def units(self):
-        return [u['_id'].rsplit('#', 1)[1] for u in self._unit_ids]
+    def unit_ids(self):
+        return [u.rsplit('#', 1)[1] for u in self._unit_rel_ids()]
 
-    def _unit_ids(self):
+    def _unit_rel_ids(self):
         r_id = 'r#%d#' % self['id']
-        # alternatively query services, and query units.
-        return self.db.settings.find(
-            {'_id': {'$regex': '^%s.*' % r_id}}, {"_id": 1})
+        # This can be fulfilled from index, but a prefix begin/end
+        # would be faster.
+        return [u['_id'] for u in self.db.settings.find(
+            {'_id': {'$regex': '^%s.*' % r_id}}, {"_id": 1})]
+
+    def history(self):
+        # Won't include deleted units, would need to iterate
+        # full set of units based on svc seq.
+        q = self.db.txns.find(
+            {"o.c": "settings", 'o.d': {'$in': self._unit_rel_ids()}})
+        for t in q:
+            Txn.format(t)
 
 
 class Machine(Entity):
